// Adapted from WojtaZam https://www.shadertoy.com/view/XdcXzn
#version 330 core
in vec2 uv; 
out vec4 outColor;

uniform sampler2D primaryScene;

uniform float saturation = 1.3;
uniform float brightness = 0.0;
uniform float contrast = 1.15;

uniform float rBal = 0.0;
uniform float gBal = 0.0;
uniform float bBal = 0.0;


mat4 brightnessMatrix( float brightness )
{
    return mat4( 1, 0, 0, 0,
                 0, 1, 0, 0,
                 0, 0, 1, 0,
                 brightness, brightness, brightness, 1 );
}

mat4 contrastMatrix( float contrast )
{
	float t = ( 1.0 - contrast ) / 2.0;
    
    return mat4( contrast, 0, 0, 0,
                 0, contrast, 0, 0,
                 0, 0, contrast, 0,
                 t, t, t, 1 );

}

mat4 saturationMatrix( float saturation )
{
    vec3 luminance = vec3( 0.3086, 0.6094, 0.0820 );
    
    float oneMinusSat = 1.0 - saturation;
    
    vec3 red = vec3( luminance.x * oneMinusSat );
    red+= vec3( saturation, 0, 0 );
    
    vec3 green = vec3( luminance.y * oneMinusSat );
    green += vec3( 0, saturation, 0 );
    
    vec3 blue = vec3( luminance.z * oneMinusSat );
    blue += vec3( 0, 0, saturation );
    
    return mat4( red,     0,
                 green,   0,
                 blue,    0,
                 0, 0, 0, 1 );
}

vec4 rebalance(vec4 col) {
    vec4 balanced = vec4(col);
    balanced.r += rBal;
    balanced.g += gBal;
    balanced.b += bBal;
    return balanced;
}

void main()
{
    vec4 color = texture( primaryScene, uv );
    color = rebalance(color);
    
	outColor = brightnessMatrix( brightness ) *
        		contrastMatrix( contrast ) * 
        		saturationMatrix( saturation ) *
        		color;
}